﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.IO;
using TwinCAT.Measurement.Scope.API.Model;
using TwinCAT.Scope2.Communications;
using TwinCAT.Measurement.Scope.Control;

namespace WindowsFormsApp1
{
    public class ScopeOperateClass
    {
        public ScopeProjectPanel scopeProjectPanel
        {
            get { return _scopeProjectPanel; }
            set
            {
                _scopeProjectPanel = value;
            }
        }
        public string ScopeFilePath
        {
            get { return _scopeFilePath; }
            set
            {
                _scopeFilePath = value;
            }
        }

        public string Status
        {
            get { return _status; }
        }

        private ScopeProjectPanel _scopeProjectPanel;
        private ScopeProject _scopeProject;
        private string _scopeFilePath;
        private string _status;
        private bool _fileLoaded;


        public ScopeOperateClass()
        {
            _scopeProjectPanel = new ScopeProjectPanel();
            _scopeProject = new ScopeProject();
            _scopeFilePath = "";
            _fileLoaded = false;
            _status = "";
        }
        public void Dispose()
        {
            try
            {
                if (_scopeProjectPanel != null && _scopeProject != null)
                {
                    if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Record)
                    {
                        _scopeProject.StopRecord();
                    }
                    if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Reply)
                    {
                        _scopeProject.Disconnect();
                    }
                }

                _fileLoaded = false;
                _status = "";
            }
            catch (Exception exp)
            {

            }
            
        }
        //加载Scope文件
        public bool LoadScopeFile()
        {
            bool result = false;
            _status = "";
            try
            {
                FileInfo fInfo = new FileInfo(_scopeFilePath);
                if (fInfo.Exists)
                {
                    //delete old configuration
                    _scopeProjectPanel.ScopeProject.Dispose();

                    //load configuration
                    _scopeProject = ScopeProject.LoadScopeProject(_scopeFilePath);
                    _scopeProjectPanel.ScopeProject = _scopeProject;
                    _status = "文件加载成功";
                    _fileLoaded = true;
                    result = true;
                }
                else
                {
                    _status = "文件不存在!";
                    result = false;
                }
           
            }
            catch (Exception exp)
            {
                _status = exp.ToString();
            }
            return result;
        }
        //开始记录
        public bool StartRecord()
        {
            bool result = false;
            _status = "";
            if (!_fileLoaded)
            {
                _status = "文件没有加载";
                return false;
            }
           try
            {
                if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Reply)
                    _scopeProject.Disconnect();

                //start record
                if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Config)
                    _scopeProject.StartRecord();
                _status = "开始记录";
                result = true;
            }
            catch (Exception exp)
            {
                _status = exp.ToString();
            }

            return result;
        }
        //停止记录
        public bool StopRecord()
        {
            bool result = false;
            if (!_fileLoaded)
            {
                _status = "文件没有加载";
                return false;
            }
            try
            {
                if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Record)
                {
                    _scopeProject.StopRecord();
                    _status = "停止记录";
                    result = true;
                }
                
            }
            catch (Exception exp)
            {
                _status = exp.ToString();
            }
            return result;
        }
        //继续显示
        public bool StartDisplay()
        {
            bool result = false;
            if (!_fileLoaded)
            {
                _status = "文件没有加载";
                return false;
            }
            try
            {               
                if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Record)
                {
                    _scopeProject.SubMember.OfType<Chart>().First().StartDisplay();
                    _status = "继续记录";
                    result = true;
                }
               else
                {
                    _status = "只有运行状态下才可以继续记录";
                }
            }
            catch (Exception exp)
            {
                _status = exp.ToString();
            }
            return result;
        }
        //暂停显示
        public bool  StopDisplay()
        {
            bool result = false;
            if (!_fileLoaded)
            {
                _status = "文件没有加载";
                return false;
            }
            try
            {
                if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Record)
                {
                    _scopeProject.SubMember.OfType<Chart>().First().StopDisplay();
                    _status = "已暂停";
                    result = true;
                }
                else
                {
                    _status = "只有运行状态下才可以暂停记录";
                }

            }
            catch (Exception exp)
            {
                _status = exp.ToString();
            }
            return result;
        }
        //数据导出CSV文件
        public bool  ExportCsv(string csvFilePath)
        {
            bool result = false;
            if (!_fileLoaded)
            {
                _status = "文件没有加载";
                return false;
            }
            try
            {
                if (_scopeProject.ScopeState == TwinCAT.Measurement.Scope.API.ScopeViewState.Reply)
                {                   
                    _scopeProject.Export(csvFilePath);
                    _status = "文件导出成功";
                    result = true;
                }
            }
            catch (Exception exp)
            {
                _status = exp.ToString();
            }
            return result;
        }
        //设置Axis的可见性
        public bool SetAxisVisibility(string AxisName,bool bVisible)
        {
            bool result = false;
            if (!_fileLoaded)
            {
                _status = "文件没有加载";
                return false;
            }
            try
            {
                Chart chart= _scopeProject.SubMember.OfType<Chart>().First();
                int axisCount = chart.SubMember.OfType<AxisGroup>().Count();
                AxisGroup axisGroup = new AxisGroup();
                bool bFind = false;
                for(int i=0;i<axisCount;i++)
                {
                    axisGroup = chart.SubMember.OfType<AxisGroup>().ElementAt(i);
                    if(axisGroup.Name== AxisName.Trim())
                    {
                        bFind = true;
                        break;
                    }
                }
                if (!bFind) { return false ; }
                TimeAxis timeAxis = axisGroup.SubMember.OfType<TimeAxis>().First();
                AxisStyle timeAxisStyle = timeAxis.SubMember.OfType<AxisStyle>().First();
                timeAxisStyle.Visible = bVisible;

                ValueAxis valueAxis = axisGroup.SubMember.OfType<ValueAxis>().First();
                AxisStyle valueAxisStyle = valueAxis.SubMember.OfType<AxisStyle>().First();
                valueAxisStyle.Visible = bVisible;

                int channelCount = axisGroup.SubMember.OfType<Channel>().Count();
                for(int i=0;i<channelCount;i++)
                {
                    Channel channel = axisGroup.SubMember.OfType<Channel>().ElementAt(i);
                    ChannelStyle channelStyle = channel.SubMember.OfType<ChannelStyle>().First();
                    channelStyle.Visible = bVisible;
                }

                _status = "设置成功";
                result = true;

            }
            catch (Exception exp)
            {
                _status = exp.ToString();
            }

            return result;
        }


    }
}
